/*
    KMines (not the KDE original version though ;-) - it's just a simple KDE app demo)
    Copyright (C) 2006 Andrej Krutak

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "mainw.h"

#include <qfont.h>
#include <qapplication.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <qlayout.h>

MainWidget::MainWidget(const char *name) : QMainWindow(0, name,
			     WDestructiveClose)
{
	W=10; H=10; M=15;
	setCaption(name);
	QPopupMenu *gameMenu =  new QPopupMenu(this);
	menuBar()->insertItem("&Game", gameMenu);
	gameMenu->insertItem("&New", this, SLOT(newGame()), Key_F2);
	gameMenu->insertItem("&Settings", this, SLOT(settings()), CTRL + Key_S);
	gameMenu->insertSeparator();
	gameMenu->insertItem("&Quit", qApp, SLOT(closeAllWindows()), CTRL + Key_Q);

	menuBar()->insertSeparator();
	QPopupMenu *helpMenu = new QPopupMenu(this);
    menuBar()->insertItem("&Help", helpMenu);
    helpMenu->insertItem("&About", this, SLOT(about()), Key_F1);

	board=new Minefield(this, 10, 10, 20);
	board->setFocus();
	setCentralWidget(board);
	layout()->setResizeMode(QLayout::Fixed);
	sdlg=new Settings;
	connect(sdlg, SIGNAL(closing()), this, SLOT(settingsClosed()));
}

void MainWidget::newGame()
{
	if (!askQuit()) return;
	setCentralWidget(0);
	delete board;

	board=new Minefield(this, W, H, M);
	board->show();
	board->setFocus();
	setCentralWidget(board);
}

void MainWidget::settings()
{
	sdlg->sW=W;
	sdlg->sH=H;
	sdlg->sM=M;
	sdlg->setValues();
	sdlg->show();
}

void MainWidget::settingsClosed()
{
	W=sdlg->sW;
	H=sdlg->sH;
	M=sdlg->sM;
	newGame();
}

void MainWidget::about()
{
	QMessageBox::about(this, "About KMines",
		"This is the game KMines. (C)2006 Andrej Krutak, distribute under GPL 2.0 or later ;-)");
}

void MainWidget::closeEvent(QCloseEvent *ce)
{
	if (askQuit()) ce->accept();
	else ce->ignore();
}

bool MainWidget::askQuit()
{
	if (!board || !board->Playing()) return true;

	return ((QMessageBox::information(this, "Question", "Do you really want to stop current game?", "Yes", "No", 0, 1))==0?true:false);
}
