/*
    KMines (not the KDE original version though ;-) - it's just a simple KDE app demo)
    Copyright (C) 2006 Andrej Krutak

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _MINEFIELD_H_
#define _MINEFIELD_H_

#include <qwidget.h>
#include <qpushbutton.h>
#include <qgrid.h>
#include <qmessagebox.h>

enum {
	CELL_MINE=1,
	CELL_FLAG=2,
	CELL_UNSURE=4,
	CELL_SHOW=8
};

class Minefield;

class Cell: public QPushButton {
	Q_OBJECT
public:
	Cell(Minefield *_mf, QWidget *parent, int _x, int _y);
	virtual ~Cell() {}

	int state, x, y;
private slots:
	void clicked();
protected:
	void mouseReleaseEvent(QMouseEvent *e);
private:
	Minefield *mf;
};

class Minefield : public QGrid {
    Q_OBJECT
public:
    Minefield(QWidget *parent = 0, int w=10, int h=10, int m=20);
	virtual ~Minefield();
	bool Playing();
	void SetPlaying();
	void ItemClicked(int x, int y);
	void ItemRightclicked(int x, int y);
private slots:

private:
	Cell ***cells; //w x h
	int W, H, M;
	bool playing;

	int explore(int x, int y) {
		if (!inCells(x, y) || (cells[y][x]->state&CELL_SHOW)==CELL_SHOW) {
			return 0;
		}

		if ((cells[y][x]->state&CELL_MINE)==CELL_MINE) {
			cells[y][x]->state=CELL_SHOW|CELL_MINE;
			return 1;
		}
		cells[y][x]->state=CELL_SHOW;
		cells[y][x]->setDown(true);
		int ma=minesAround(x, y);
		if (ma>0) {
			char buf[10];
			sprintf(buf, "%d", ma);
			cells[y][x]->setText(buf);
		} else {
			cells[y][x]->setText(" ");
			explore(x-1, y-1);
			explore(x+1, y+1);
			explore(x+1, y-1);
			explore(x-1, y+1);
			
			explore(x-1, y);
			explore(x+1, y);
			explore(x, y-1);
			explore(x, y+1);
		}
		return 0;
	}

	int isMine(int x, int y) {
		if (!inCells(x, y)) return 0;
		else return ((cells[y][x]->state&CELL_MINE)==CELL_MINE)?1:0;
	}

	bool inCells(int x, int y) {
		return (x>=0 && y>=0 && x<W && y<H);
	}

	int minesAround(int x, int y) {
		return
			isMine(x-1, y-1)+isMine(x-1, y)+isMine(x-1, y+1)+
			isMine(x, y-1)+isMine(x, y)+isMine(x, y+1)+
			isMine(x+1, y-1)+isMine(x+1, y)+isMine(x+1, y+1);
	}

	bool isWon() {
		int x, y;
		
		for (y=0; y<H; y++) {
			for (x=0; x<W; x++)
				if ((cells[y][x]->state&CELL_MINE)!=CELL_MINE && (cells[y][x]->state&CELL_SHOW)!=CELL_SHOW)
					return false;
		}
		return true;
	}

	void gameOver(int how);
};

#endif // _MINEFIELD_H_
